@@ -1,23 +1,21 @@
 #!/usr/bin/env python
 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 3
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 """
-    smallrnaseq implementation of command line functionality
-    Created November 2016
-    Copyright (C) Damien Farrell
-
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License
-    as published by the Free Software Foundation; either version 3
-    of the License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+Module for running the command line application of smallrnaseq.
 """
 
 from __future__ import absolute_import, print_function
@@ -25,6 +23,15 @@ import sys, os, string, types, shutil, time
 import glob
 import pandas as pd
 from smallrnaseq import config, base, analysis, utils, aligners, novel, plotting, de
+snap_msg = 'when running from a snap you should use your home folder for reading/writing'
+home = os.path.expanduser('~')
+config_path = os.path.join(home,'.config','smallrnaseq')
+
+if not os.path.exists(config_path):
+    try:
+        os.makedirs(config_path, exist_ok=True)
+    except:
+        os.makedirs(config_path)
 
 class WorkFlow(object):
     """Class for implementing a rna/mirna workflow from a set of options"""
@@ -42,19 +49,30 @@ class WorkFlow(object):
         if self.filenames != '':
             self.files = self.filenames.split(',')
         elif self.path != '':
-            self.files = glob.glob(os.path.join(self.path,'*.fastq'))
+            self.files = glob.glob(os.path.join(self.path,'*.f*q'))
             if len(self.files) == 0:
                 print ('no fastq files found')
+                if check_snap() == True:
+                    print (snap_msg)
                 return False
         else:
             print ('you should provide at least one file or folder')
+            if check_snap() == True:
+                print (snap_msg)
             return False
         self.files = [str(f) for f in self.files]
+        if self.adapter != '':
+            trim_path = os.path.join(self.output, 'trimmed')
+            os.makedirs(trim_path, exist_ok=True)
+            self.files = base.trim_files(self.files, trim_path, self.adapter)
+
         aligners.BOWTIE_INDEXES = aligners.SUBREAD_INDEXES = self.index_path
         if self.default_params != '':
             aligners.set_params(self.aligner, self.default_params)
         if not os.path.exists(self.index_path):
             print ('no such folder for indexes')
+            if check_snap() == True:
+                print (snap_msg)
             return False
         if not os.path.exists(self.output):
             os.mkdir(self.output)
@@ -88,6 +106,15 @@ class WorkFlow(object):
         else:
             return True
 
+    def save_samples(self):
+        """Save sample info"""
+
+        s = self.samples
+        #df = pd.DataFrame(list(self.labels.items()),columns = ['name','id'])
+        #print (df)
+        s.to_csv(os.path.join(self.output,'samples.csv'),float_format='%4f')
+        return
+
     def run(self):
         """Execute the predefined workflows"""
 
@@ -97,6 +124,8 @@ class WorkFlow(object):
             self.map_libraries()
         if self.features != '':
             self.map_genomic_features()
+        df = self.save_samples()
+        print ('done')
         print ('intermediate files saved to %s' %self.temp_path)
         return
 
@@ -159,6 +188,7 @@ class WorkFlow(object):
         ref_name = self.ref_name
         mat_name = 'mirbase-%s' %self.species
         self.aligner_params[mat_name] = self.mirna_params
+        novel.VERBOSE = self.verbose
 
         if self.check_index(ref_name) == False:
             print ('no index for reference genome')
@@ -169,8 +199,10 @@ class WorkFlow(object):
                                        species=self.species, ref_genome=ref_name,
                                        pad5=self.pad5, pad3=self.pad3, aligner=self.aligner,
                                        samplelabels=self.labels,
-                                       params=self.aligner_params)
+                                       params=self.aligner_params,
+                                       verbose=self.verbose)
 
+        self.results = res
         #seperate out mature counts and save
         matcounts = counts[counts.ref==mat_name]
         res.to_csv( os.path.join(out, 'results.csv'),index=False )
@@ -178,19 +210,30 @@ class WorkFlow(object):
         matcounts.to_csv( os.path.join(out, 'mirbase_mature_counts.csv'), index=False,
                             float_format='%.1f' )
         counts.to_csv( os.path.join(out, 'all_counts.csv'), index=False, float_format='%.1f')
-        plot_results(res, out)
+
+        #get fractions per sample and plot results
+        c = base.pivot_count_data(res, idxcols=['name','ref'])
+        self.samples = s = base.get_fractions_mapped(res)
+        print (s)
+        plot_results(s, c, out)
 
         #isomir counting
         print ()
         print ('counting isomirs..')
         iso, isocounts = base.map_isomirs(self.files, temp, self.species,
                                           samplelabels=self.labels)
-        isocounts.to_csv( os.path.join(out, 'isomir_counts.csv'), index=False, float_format='%.1f')
-
+        if isocounts is not None:
+            isocounts.to_csv( os.path.join(out, 'isomir_counts.csv'),
+                                index=False, float_format='%.1f')
+        else:
+            print ('no isomirs could be counted')
         #novel prediction
+        #train classifier first if not present
+        novel.create_classifier()
+
         if self.ref_fasta == '' or not os.path.exists(self.ref_fasta):
             print ('no reference genome file, skipping novel mirna step')
-        elif ref_name == None:
+        elif ref_name == None or ref_name == '':
             print ('no index for ref genome, required for novel mirna step')
         elif check_viennarna() == False:
             print ('Vienna RNA package not installed')
@@ -204,13 +247,15 @@ class WorkFlow(object):
             allreads = utils.combine_aligned_reads(temp, idx=ref_name)
             new,cl = novel.find_mirnas(allreads, self.ref_fasta, species=self.species,
                                        score_cutoff=float(self.score_cutoff),
-                                       read_cutoff=int(self.read_cutoff))
+                                       read_cutoff=int(self.read_cutoff),
+                                       cpus=self.cpus)
+            if new is None or len(new) == 0:
+                print ('Could not find any novel mirnas.')
+                print ('There may not be sufficient aligned reads or the score cutoff is too high.\n')
+                return
             if self.strict == True:
                 new = new[new.mature_check=='ok']
                 print ('filtered %s' %len(new))
-            if new is None or len(new) == 0:
-                print ('could not find any novel mirnas at this score cutoff')
-                return
             new.to_csv(os.path.join(out,'novel_mirna.csv'), index=False)
 
             #pad mature novel and write to fasta for counting
@@ -266,24 +311,25 @@ def check_viennarna():
     except ImportError as e:
         return False
 
-def plot_results(res, path):
+def plot_results(df, counts, path):
     """Some results plots"""
 
-    if res is None or len(res) == 0:
+    if df is None or len(df) == 0:
         return
-    counts = base.pivot_count_data(res, idxcols=['name','ref'])
-    x = base.get_fractions_mapped(res)
-    print (x)
-    fig = plotting.plot_fractions(x)
+
+    import seaborn as sns
+    sns.set_style('white')
+    sns.set_context("paper",font_scale=1.2)
+    fig = plotting.plot_fractions(df)
     fig.savefig(os.path.join(path,'libraries_mapped.png'))
     fig = plotting.plot_sample_counts(counts)
     fig.savefig(os.path.join(path,'total_per_sample.png'))
     fig = plotting.plot_read_count_dists(counts)
     fig.savefig(os.path.join(path,'top_mapped.png'))
     scols,ncols = base.get_column_names(counts)
-    for l,df in counts.groupby('ref'):
+    for l,g in counts.groupby('ref'):
         if 'mirbase' in l:
-            fig = plotting.plot_read_count_dists(df)
+            fig = plotting.plot_read_count_dists(g)
             fig.savefig(os.path.join(path,'top_%s.png' %l))
     #if len(scols)>1:
     #    fig = plotting.expression_clustermap(counts)
@@ -317,6 +363,8 @@ def diff_expression(opts):
     print()
     print ('running differential expression')
     path = opts['output']
+    if not os.path.exists(path):
+        os.makedirs(path,exist_ok=True)
     labelsfile = opts['sample_labels']
     countsfile = opts['count_file']
     logfccutoff = float(opts['logfc_cutoff'])
@@ -389,8 +437,8 @@ def diff_expression(opts):
     m = de.melt_samples(counts, labels, names, samplecol=samplecol)
     import seaborn as sns
     kind = opts['de_plot']
-    g = sns.factorplot(factorcol,'read count', data=m, col='name', kind=kind,
-                            col_wrap=5, size=3, aspect=1.2,
+    g = sns.catplot(x=factorcol,y='read count', data=m, col='name', kind=kind,
+                            col_wrap=5, height=3, aspect=1.2,
                             legend_out=True,sharey=False, order=xorder)
     deplot = os.path.join(path,'de_genes.png')
     g.savefig(deplot)
@@ -414,16 +462,23 @@ def print_help():
     print ('for further information')
     return
 
-def run_test():
+def check_snap():
+    """Check if inside a snap"""
+
+    if 'SNAP_COMMON' in os.environ:
+        return True
+    return False
+
+def test_run():
     """Run mirna test with test files"""
 
     print ('running miRNA counting test')
-    indpath = 'testing/indexes'
-    aligners.BOWTIE_INDEXES = indpath
-    if not os.path.exists(indpath):
-        os.makedirs(indpath)
+    idxpath = 'testing/indexes'
+    if check_snap() == True:
+        idxpath = os.path.join(home, idxpath)
+    aligners.BOWTIE_INDEXES = idxpath
     lib1 = os.path.join(base.datadir, 'bosTau8-tRNAs.fa')
-    aligners.build_bowtie_index(lib1, path=indpath)
+    aligners.build_bowtie_index(lib1, path=idxpath)
     f1 = os.path.join(base.datadir, 'bovine_plasma_sample.fastq')
     f2 = os.path.join(base.datadir, 'bovine_serum_sample.fastq')
     path = os.path.join('testing', 'ncrna_map')
@@ -450,21 +505,28 @@ def main():
                         help="input file", metavar="FILE")
     parser.add_option("-l", "--collapse", dest="collapse", action="store_true",
                         default=False, help="collapse reads in input file")
-    parser.add_option("-a", "--trim", dest="trim",  type='string',
+    parser.add_option("-a", "--adapter", dest="adapter",  type='string',
                         help="trim given adapter in input file")
     parser.add_option("-d", "--de", dest="de",  action="store_true",
                         default=False, help="run DE analysis")
     parser.add_option("-t", "--tests", dest="tests",  action="store_true",
                         default=False, help="run tests")
+    parser.add_option("-v", "--version", dest="version", action="store_true",
+                        help="Get version")
+
     opts, remainder = parser.parse_args()
 
     if opts.infile != None:
-        if opts.trim != None:
-            utils.trim_adapters(opts.infile, adapters=opts.trim, outfile='cut.fastq')
+        if opts.adapter != None:
+            utils.trim_adapters(opts.infile, adapters=opts.adapter, outfile='cut.fastq')
         if opts.collapse == True:
             base.collapse_reads(opts.infile)
     elif opts.build != None:
-        build_indexes(opts.build, 'indexes')
+        idxpath = 'indexes'
+        if check_snap() == True:
+            idxpath = os.path.join(home, 'indexes')
+        build_indexes(opts.build, idxpath)
+        print ('wrote index to folder %s' %idxpath)
     elif opts.config != None and os.path.exists(opts.config):
         cp = config.parse_config(opts.config)
         options = config.get_options(cp)
@@ -485,7 +547,10 @@ def main():
         else:
             print_help()
     elif opts.tests == True:
-        run_test()
+        test_run()
+    elif opts.version == True:
+        from . import __version__
+        print ('smallrnaseq version %s' %__version__)        
     else:
         if opts.config != None:
             conffile = opts.config
