@@ -1,23 +1,25 @@
 #!/usr/bin/env python
 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 3
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 """
-    Module for core smallrnaseq functions
-    Created July 2014
-    Copyright (C) Damien Farrell
+Module for core smallrnaseq functions. All mapping and gene counting methods are
+implemented here.
+Created July 2014
+Copyright (C) Damien Farrell
 
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License
-    as published by the Free Software Foundation; either version 3
-    of the License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 from __future__ import absolute_import, print_function
@@ -30,12 +32,14 @@ try:
 except:
     'HTSeq not present'
 from . import utils, aligners
+#make sure we are using the agg backend for melt_samples
+import matplotlib
+matplotlib.use('agg')
 
 path = os.path.dirname(os.path.abspath(__file__)) #path to module
 datadir = os.path.join(path, 'data')
 MIRBASE = os.path.join(datadir, 'miRBase_all.csv')
 
-
 def first(x):
     return x.iloc[0]
 
@@ -72,14 +76,16 @@ def gtf_to_dataframe(gtf=None, gtf_file=None, index='transcript_id'):
 
 def count_features(samfile, features=None, gtffile=None, readcounts=None, merge=False):
     """Count reads in features from an alignment, if no truecounts we
-       assume a non-collapsed file was used to map
+       assume a non-collapsed file was used to map.
+
        Args:
            samfile: mapped sam file
            gtffile: feature file
            features: annotations read from bed or gtf file
-           truecounts: read counts from original (un-collapsed) file
+           readcounts: read counts from original (un-collapsed) file
            merge: whether to merge the gtf fields with the results
-       Returns: dataframe of genes with total counts
+       Returns:
+           dataframe of genes with total counts
     """
 
     if gtffile != None:
@@ -151,6 +157,7 @@ def count_aligned(samfile, collapsed=None, readcounts=None, by='name',
                    count_method='split'):
     """Count short read alignments from a sam or bam file. Designed to be used with
        collapsed reads with original read counts in fasta id.
+
        Args:
            samfile: mapped sam file
            collapsed: collapsed fasta with original read counts in name
@@ -164,17 +171,17 @@ def count_aligned(samfile, collapsed=None, readcounts=None, by='name',
         readcounts = utils.read_collapsed_file(collapsed)
 
     counts = utils.get_aligned_reads(samfile, readcounts=readcounts)
+    if len(counts) == 0:
+        return pd.DataFrame()
     if count_method == 'split':
         #split multi mapped sequence counts evenly
         counts['reads'] = counts.groupby(['seq'])['reads'].transform(lambda x: x / len(x))
 
-    #if drop_duplicates == True:
-    #    counts = counts.drop_duplicates('seq')
     total = readcounts.reads.sum()
     counts = counts.groupby('name').agg({'reads':np.sum}).reset_index()
     mapped = float(counts.reads.sum())
     if len(counts) > 0:
-        print ('%s/%s reads counted, %.2f percent' %(mapped, total, mapped/total*100))
+        print ('%s/%s reads counted, %.3f percent' %(mapped, total, mapped/total*100))
     else:
         print ('no counts found')
     return counts
@@ -182,6 +189,7 @@ def count_aligned(samfile, collapsed=None, readcounts=None, by='name',
 def pivot_count_data(counts, idxcols='name', norm_method='library', sortby=None):
     """Pivot read counts created by count_aligned over multiple samples
        and get normalised read counts.
+
        Args:
            counts: dataframe of raw count data with samples per column
            idxcols: name of index column
@@ -204,7 +212,7 @@ def pivot_count_data(counts, idxcols='name', norm_method='library', sortby=None)
     x = x.join(n)
     #scols,ncols = get_column_names(x)
     x['total_reads'] = x[scols].sum(1)
-    x['mean_norm'] = x[ncols].apply(lambda r: r[r.nonzero()[0]].mean(),1)
+    x['mean_norm'] = x[ncols].apply(lambda r: r[r.to_numpy().nonzero()[0]].mean(),1)
     x = x.reset_index()
     #print (x[50:])
     if sortby != None:
@@ -213,7 +221,8 @@ def pivot_count_data(counts, idxcols='name', norm_method='library', sortby=None)
 
 def normalize_samples(counts, norm_method='library', rename=True):
     """Normalize over a matrix of samples explicitly, this will overwrite any 'norm'
-       columns created previously when pivoting the count data
+       columns created previously when pivoting the count data.
+
        Args:
             counts: dataframe of raw count data with samples per column
             rename: rename columns with 'norm' label and add to existing ones
@@ -241,17 +250,21 @@ def get_column_names(df):
     cols = [i.split(' ')[0] for i in ncols if i not in ignore]
     return cols, ncols
 
-def total_library_normalize(df):
+def total_library_normalize(df,factor=1e6):
     """Normalise by size of total reads"""
 
     df = df.copy()
     for col in df:
-        df[col] = df[col]/df[col].sum()*1e6
-    df = df.round(2)
+        try:
+            df[col] = df[col]/df[col].sum()*factor
+        except:
+            df[col] = df[col]
+            print ('not a numerical column')
+    #df = df.round(2)
     return df
 
 def quantile_normalize(df):
-    """Quantile normlization of counts for multiple samples.
+    """Quantile normalization of counts for multiple samples.
        see https://github.com/ShawnLYU/Quantile_Normalize
     """
 
@@ -283,14 +296,14 @@ def deseq_normalize(df):
     df = df.apply(lambda r: r/r.mean(),1)
     return df
 
-def map_rnas(files, indexes, outpath, collapse=True, adapters=None, aligner='bowtie',
+def map_rnas(files, indexes, outpath, collapse=True, aligner='bowtie',
              norm_method='library', use_remaining=True, overwrite=False,
-             samplelabels=None, params={}, count_method='split'):
-    """Map reads to one or more gene annotations, assumes adapters are removed
+             samplelabels=None, params={}, count_method='split', cpus=2, verbose=True):
+    """Map reads to one or more gene annotations, assumes adapters are removed.
+
     Args:
         files: input fastq read files
         indexes: bowtie indexes of annotations/genomes
-        adapters: if adapters need to be trimmed
         use_remaining: only align to remaining reads after each index
         overwrite: whether to overwrite temp files
         samplelabels: mapping of file names to short ids
@@ -321,12 +334,14 @@ def map_rnas(files, indexes, outpath, collapse=True, adapters=None, aligner='bow
     for cfile in cfiles:
         rem = None
         filename = os.path.splitext(os.path.basename(cfile))[0]
-        print (cfile)
+        print ('sample: \033[1m%s\033[0m' %filename)
+        print ('collapsed file: %s' %cfile)
+        print ()
         readcounts = utils.read_collapsed_file(str(cfile))
         total = readcounts.reads.sum()
-        print (filename)
+
         for idx in indexes:
-            print (idx)
+            print ('mapping to %s' %idx)
             if idx in params:
                 align_params = params[idx]
             else:
@@ -338,11 +353,11 @@ def map_rnas(files, indexes, outpath, collapse=True, adapters=None, aligner='bow
             else:
                 query = cfile
             samfile = str(os.path.join(outpath, '%s_%s.sam' %(filename,idx)))
-            rem = os.path.join(outpath, filename+'_r.fa')
+            rem = os.path.join(outpath, filename+'_'+idx+'_r.fa')
 
             if aligner == 'bowtie':
-                aligners.bowtie_align(query, idx, outfile=samfile,
-                                      remaining=rem, verbose=True)
+                aligners.bowtie_align(query, idx, outfile=samfile, cpus=cpus,
+                                      remaining=rem, verbose=verbose)
             elif aligner == 'subread':
                 aligners.subread_align(query, idx, samfile)
 
@@ -359,23 +374,26 @@ def map_rnas(files, indexes, outpath, collapse=True, adapters=None, aligner='bow
             #print (counts.reads.sum(), total)
             result.append(counts)
             print()
-        print()
+        print('--------------------------')
     if len(result) == 0:
         return
     result = pd.concat(result)
     counts = pivot_count_data(result, idxcols=['name','ref'], norm_method=norm_method,
                               sortby='total_reads')
-    print ('done')
+    #print ('done')
     return result, counts
 
 def map_genome_features(files, ref, gtf_file, outpath='', aligner='bowtie',
                         overwrite=True, aligner_params=''):
-    """Map multiple files to a genome with features and return/process hits.
-       Can be used for miRNA discovery
+    """Convenience method that maps multiple files to a genome with features
+       and return/process hits. Can be used for miRNA discovery.
+
        Args:
            ref: genome bowtie index name
            gtf_file: gtf or bed file with features
-           bowtie _index: path with bowtie indexes
+           outpath: output path
+           aligner: short read alligner to use
+           aligner_params: aligner parameters
     """
 
     if aligner_params != '' :
@@ -439,27 +457,36 @@ def assign_sample_ids(names, outfile='sample_labels.csv'):
     l.to_csv(outfile)
     return labels
 
-def trim_files(files, outpath, adapters):
+def trim_files(files, outpath, adapter):
     """Trim adapters from fastq files"""
 
+    trimmed = []
     for f in files:
-        cut = os.path.join(outpath, f)
+        cut = os.path.join(outpath, os.path.basename(f))
         if not os.path.exists(cut):
-            trim_adapters(f, adapters, cut)
-    return
+            utils.trim_adapters(f, adapter, cut)
+        trimmed.append(cut)
+    return trimmed
 
-def collapse_reads(infile, outfile=None, min_length=15, progress=False):
-    """Collapse identical reads, retaining copy number in a csv file
-       and writing collapsed reads to a new fasta file.
-       Creates a collapsed fasta file of unique reads and a csv
-       file with the """
+def collapse_reads(infile, outfile=None, min_length=15):
+    """Collapse identical reads, writing collapsed reads to a new fasta file.
+      Retains copy number in fasta headers. Each sequence in the resulting file
+      should be unique.
+
+    Args:
+        infile: input fastq file
+        outfile: output fasta file with collapsed reads
+        min_length: minimum length of read to include
+    Returns:
+        True if successful, otherwise False
+    """
 
     #from itertools import islice
     if outfile == None:
         outfile = os.path.splitext(infile)[0]+'_collapsed.fa'
     print ('collapsing reads %s' %infile)
     ext = os.path.splitext(infile)[1]
-    if ext == '.fastq':
+    if ext == '.fastq' or ext == '.fq':
         fastfile = HTSeq.FastqReader(infile, "solexa")
     elif ext == '.fa' or ext == '.fasta':
         fastfile = HTSeq.FastaReader(infile)
@@ -495,8 +522,8 @@ def collapse_reads(infile, outfile=None, min_length=15, progress=False):
     return True
 
 def collapse_files(files, outpath, **kwargs):
-    """Collapse reads and save counts
-        min_length: min length of reads to include
+    """Collapse reads and save counts. kwargs are passed to collapse_reads
+       method.
     """
 
     outfiles = []
@@ -546,6 +573,7 @@ def get_mirbase(species=None):
 
 def get_mirbase_sequences(species='hsa', pad5=0, pad3=0, dna=False):
     """Extract species specific sequences from mirbase file.
+
        Args:
            species: 3-letter code for species
            n: bases to extend around ends of mature sequence
@@ -569,7 +597,8 @@ def get_mirbase_sequences(species='hsa', pad5=0, pad3=0, dna=False):
 
 def build_mirbase_index(species, aligner='bowtie', pad5=3, pad3=5,
                         kind='mature', index_path='indexes'):
-    """Build species-specific mirbase bowtie index
+    """Build species-specific mirbase bowtie index.
+
        Args:
            species: 3-letter code for species
            aligner: which aligner to build for
@@ -587,7 +616,7 @@ def build_mirbase_index(species, aligner='bowtie', pad5=3, pad3=5,
         idxname = 'precursors-'+species
         outfile = '%s.fa' %idxname
         utils.dataframe_to_fasta(mirs, seqkey='precursor', idkey='mirbase_id',
-                                outfile=outfile)
+                                    outfile=outfile)
         print ('got %s sequences' %len(mirs))
 
     if aligner == 'bowtie':
@@ -602,9 +631,11 @@ def map_mirbase(files, species='bta', outpath='mirna_results', indexes=[],
                 pad5=3, pad3=5, **kwargs):
     """Map multiple fastq files to mirbase mature sequences and get
        count results into one file. Used for counting of known miRNAs.
+
        Args:
-           Species: three letter name of species using mirbase convention
+           species: three letter name of species using mirbase convention
            indexes: other libraries to align to before we count mirnas
+           outpath: output folders
     """
 
     if not os.path.exists(outpath):
@@ -643,6 +674,8 @@ def map_isomirs(files, outpath, species, samplelabels=None):
         #countsfile = os.path.join(outpath, '%s.csv' %filename)
         collapsed = os.path.join(outpath, '%s.fa' %filename)
         c = count_isomirs(samfile, collapsed, species)
+        if c is None:
+            return None, None
         if samplelabels != None:
             c['label'] = samplelabels[filename]
         else:
@@ -656,8 +689,8 @@ def _get_iso_class(x, refs, crefs):
     """Get isomir class using sRNAbench type scheme"""
 
     name = x['name']
-    ref = refs.ix[name]
-    cseq = crefs.ix[name].sequence
+    ref = refs.loc[name]
+    cseq = crefs.loc[name].sequence
     cstart = utils.find_subseq(ref.sequence, cseq)
     cend = cstart+len(cseq)
     start = utils.find_subseq(ref.sequence, x.seq)
@@ -702,6 +735,8 @@ def count_isomirs(samfile, collapsed, species):
     reads = utils.get_aligned_reads(samfile, collapsed)
     reads = reads.drop(['read_id','start','end'],1)
     x = reads.apply(lambda x: _get_iso_class(x, mirs, canonical),1)
+    if len(x) == 0:
+        return
     reads = reads.join(x)
     reads['name'] = reads.name + '_' + reads.subclass
     return reads
