@@ -27,7 +27,7 @@ import subprocess
 #import pylab as plt
 import numpy as np
 import pandas as pd
-from . import base, utils
+#from . import base, utils
 
 mirdeep2options = {'base': [('input',''),('adapter','TGGAATTCTCGGGTGCCAAGG'),('filetype','fastq'),
                     ('bowtieindex',''),('refgenome',''),('species','hsa'),
@@ -216,7 +216,8 @@ def read_results_file(infile):
     colstorename = {'example miRBase miRNA with the same seed':'mirbase seed match',
          'significant randfold p-value': 'randfold'}
     df = df.rename(columns=colstorename)
-    df = df.convert_objects(convert_numeric=True)
+    #df = df.convert_objects(convert_numeric=True)
+    df = df.infer_objects()
     #df['chr'] = df['provisional id'].apply(get_chromosome)
     df['seed'] = df['consensus mature sequence'].apply(lambda x: x[1:8])
     coords = df['precursor coordinate'].apply(get_coords)
@@ -264,6 +265,7 @@ def get_results(path):
         res.append(q)
     res = pd.concat(res)
 
+    res = res.apply( lambda x: pd.to_numeric(x, errors='ignore' ))
     #get mean normalised count
     res['mean_norm'] = res.filter(regex="norm").apply(lambda r: r[r.nonzero()[0]].mean(),1)
     res = res.sort_values(by=['read_count'], ascending=False)
