@@ -1,27 +1,27 @@
 #!/usr/bin/env python
 
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 3
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 """
-    Misc miRNA analysis routines
+    Misc utility routines for smallrnaseq.
     Created June 2014
     Copyright (C) Damien Farrell
-
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License
-    as published by the Free Software Foundation; either version 3
-    of the License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 from __future__ import absolute_import, print_function
-import sys, os, string, types, re, csv
+import sys, os, string, types, re, csv, time
 import shutil, glob, collections
 import itertools
 from itertools import islice
@@ -64,6 +64,63 @@ def gzipfile(filename, remove=False):
         os.remove(filename)
     return
 
+def _run_multiprocess(recs, cpus=2, worker=None, **kwargs):
+    """
+    Call function with multiprocessing pools. Used for running predictions
+    in parallel where the main input is a pandas dataframe.
+    Args:
+        recs: input dataframe or iterable
+        cpus: number of cores to use
+        worker: function to be run in parallel, expected to return a dataframe
+    Returns:
+        concatenated result, a pandas dataframe
+       """
+
+    import multiprocessing as mp
+    maxcpu = mp.cpu_count()
+    if cpus == 0 or cpus > maxcpu:
+        cpus = maxcpu
+    if cpus >= len(recs):
+        cpus = len(recs)
+    pool = mp.Pool(cpus)
+    funclist = []
+    st = time.time()
+    chunks = np.array_split(recs,cpus)
+    #print ([len(i) for i in chunks])
+
+    for recs in chunks:
+        f = pool.apply_async(worker, [recs,kwargs])
+        funclist.append(f)
+    result = []
+    try:
+        for f in funclist:
+            df = f.get(timeout=None)
+            #print (df)
+            if df is not None and len(df)>0:
+                result.append(df)
+    except KeyboardInterrupt:
+        print ('process interrupted')
+        pool.terminate()
+        sys.exit(0)
+    pool.close()
+    pool.join()
+
+    if type(result) is list:
+        #if multiple dfs returned from the function we zip, concat
+        #and put back in a list
+        result = zip(*result)
+        r=[]
+        for df in result:
+            r.append(pd.concat(df))
+        result=r
+    else:
+        if len(result)>0:
+            result = pd.concat(result)
+
+    t=time.time()-st
+    print ('took %s seconds' %str(round(t,3)))
+    return result
+
 def create_html(df,name,path='.'):
     """Create a basic html page for dataframe results"""
 
@@ -259,7 +316,7 @@ def get_mifam():
     df = pd.DataFrame(data,columns=['id','name','family'])
     return df
 
-def trim_adapters(infile, adapter, outfile='cut.fastq', method='default'):
+def trim_adapters(infile, adapter, outfile='cut.fastq', method='cutadapt'):
     """Trim adapters using cutadapt"""
 
     if not type(adapter) is str:
@@ -275,7 +332,7 @@ def trim_adapters(infile, adapter, outfile='cut.fastq', method='default'):
             new.write_to_fastq_file( newfile )
         newfile.close()
     elif method == 'cutadapt':
-        cmd = 'cutadapt -m 18 -O 5 -q 20 -a %s %s -o %s' %(adapter,infile,outfile)
+        cmd = 'cutadapt -m 18 -O 5 -q 20 -b %s %s -o %s' %(adapter,infile,outfile)
         print (cmd)
         result = subprocess.check_output(cmd, shell=True, executable='/bin/bash')
     return
@@ -414,19 +471,24 @@ def features_to_gtf(df, filename):
     return gtf
 
 def sequence_from_coords(fastafile, coords):
-    """Fasta sequence from genome coords"""
+    """Fasta sequence from genome coords.
+    Args:
+        fastafile: inpout fasta file
+        coords: genome coordinates as tuple of the form
+                (chrom,start,end,strand)
+    """
 
-    from pybedtools import BedTool
-    chrom,start,end,strand = coords
     if not os.path.exists(fastafile):
         print ('no such file')
         return
+    chrom,start,end,strand = coords
+    from pyfaidx import Fasta
+    genes = Fasta(fastafile)
     try:
-        if strand == '+':
-            seq = str(BedTool.seq(coords, fastafile))
-        else: #reverse strand
-            seq = str(BedTool.seq(coords, fastafile))
-            seq = str(HTSeq.Sequence(seq).get_reverse_complement())
+        #gets seq string from genome
+        seq = str(genes[chrom][start:end])
+        if strand == '-':
+            seq = str(HTSeq.Sequence(seq.encode()).get_reverse_complement())
     except Exception as e:
         print (e)
         return
@@ -486,7 +548,10 @@ def read_collapsed_file(collapsed):
     #original read counts are encoded in fasta names
     df = fasta_to_dataframe(collapsed).reset_index()
     #original count stored in id as second value
-    df['read_id'], df['reads'] = df.name.str.split('_', 1).str
+    #df['read_id'], df['reads'] = df.name.str.split('_', 1).str
+    df2 = df.name.str.split('_', 1, expand=True)
+    df2.columns = ['read_id','reads']
+    df = df.join(df2)
     df['reads'] = df.reads.astype(int)
     df['read_id'] = df.read_id.astype(int)
     df = df.drop(['name','description'],1)
@@ -534,6 +599,7 @@ def combine_aligned_reads(path, idx, filenames=None):
         #print(f)
         name = os.path.splitext(os.path.basename(f))[0]
         samfile = os.path.join(path, '%s_%s.sam' %(name,idx))
+        #print (samfile)
         if not os.path.exists(samfile):
             print ('no sam file')
             continue
@@ -704,9 +770,15 @@ def get_bg(seq, struct=None):
     """
 
     import forgi.graph.bulge_graph as cgb
+    seq = seq.replace('T','U')
     if struct == None:
-        struct,sc = utils.rnafold(seq)
+        struct,sc = rnafold(seq)
     bg = cgb.BulgeGraph()
     bg.from_dotbracket(struct)
-    bg.seq = seq
-    return bg
+    #print (bg.struct)
+    try:
+        bg.seq = seq
+        return bg
+    except Exception as e:
+        print (e)
+        return
